----------------------------------------------------------------------------------
-- Company: Boston University EDF
-- Engineer: Christopher Woodall
-- 
-- Create Date:    17:01:59 06/26/2012 
-- Design Name: 
-- Module Name:    TTCDemux - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;

entity ttc_demux is

  port (
    -- Inputs
    input_stream : in std_logic;
    clk          : in std_logic;
    clk_en       : in std_logic;
    rst_n        : in std_logic;        -- Low Active Reset

    -- Outputs
    chA_data : out std_logic;
    chA_rdy  : out std_logic;           -- change to chA_strobe
    chB_data : out std_logic;
    chB_rdy  : out std_logic            -- change to chB_strobe
    );

end ttc_demux;

architecture Behavioral of ttc_demux is
  signal   chA_counter : integer range 0 to 31 := 0;

  -- const int chA_limit: The max # of ones that can be transmitted on chA
  constant chA_limit   : integer range 0 to 31 := 11;  

  -- std_logic current_ch: toggled back and forth. By default 
  -- channel A is assumed. The correction occurs by delaying change
  -- from '0' to '1' by an additional clock cycle. This has the result
  -- of inverting the signal. The mapping should be, once a lock has been
  -- acheived.
  --
  -- '0' => A
  -- '1' => B
  signal current_ch : std_logic := '0';
begin
  figure_out_channels : process (clk)
  begin
    if rising_edge(clk) then
      if (rst_n = '0') then
        chA_counter <= 0;
      else
        if (clk_en = '1') then
          -- If we are in Channel A ('0') and we received a '1' (L1A),
          -- add it to the counter chA_counter. We are looking for consecutive 0s
          -- Thus we clear chA_counter when we get a zero.
          --
          -- If chA_counter exceeds chA_limit (23 according to the TTC
          -- standards document) we know that what we currently think is
          -- chA is chB.
          if (current_ch = '0') then
            if (input_stream = '1') then
              if (chA_counter = chA_limit) then
                chA_counter <= 0;
                current_ch  <= '0';
              else
                chA_counter <= chA_counter + 1;
                current_ch <= current_ch xor '1';          
              end if;
            else
              current_ch <= current_ch xor '1';                      
              chA_counter <= 0;  -- If we get a 0 on channel A then we reset the counter
            end if;
          else
            current_ch <= current_ch xor '1';          
          end if;
          
        end if;
      end if;
    end if;
  end process;

  output_data : process (clk, rst_n)
  begin

    if rising_edge(clk) then
      if rst_n = '0' then
        chA_data <= '0';
        chB_data <= '0';
        chA_rdy  <= '0';
        chB_rdy  <= '0';
      elsif (clk_en = '1') then
        if (current_ch = '0') then
          chA_data <= input_stream;
          chA_rdy  <= '0';
          chB_rdy  <= '1';
        else
          chB_data <= input_stream;
          chA_rdy  <= '1';
          chB_rdy  <= '0';
        end if;
      end if;
    end if;
  end process;
end Behavioral;
