----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    13:03:54 09/07/2012 
-- Design Name: 
-- Module Name:    TTSInterface - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: UNISIM libraries
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.std_logic_misc.all;
-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity TTCInterface is
  port(
    -- Control Interface
    clk40    : in std_logic;
    clk80    : in std_logic;
    clk160   : in std_logic;
    clk320   : in std_logic;
    clk320_n : in std_logic;

    rst_n  : in std_logic;
    enable_n : in std_logic;
    -- Tx TTC Interface
    TxL1A  : in std_logic;               -- L1A inputs, sampled at clk40
    TxBC0  : in std_logic_vector(7 downto 0);  -- TxBC0, input when _wr is detected
    TxBC0_wr : in std_logic;
    TxBC0_sent : out std_logic;
    TxBC0_orbit_strobe : out std_logic;
    TxBC0_orbit_en : in std_logic;
    TxBC0_orbit_ctr : out unsigned(31 downto 0);
    
    -- Rx TTC Interface
    Rx0L1A  : out std_logic;
    Rx0L1A_clk : out std_logic;
    Rx1L1A  : out std_logic;
    Rx1L1A_clk : out std_logic;
    
    Rx0BC0  : out std_logic_vector(7 downto 0);
    Rx0BC0Err  : out std_logic_vector(2 downto 0);
    Rx1BC0  : out std_logic_vector(7 downto 0);
    Rx1BC0Err  : out std_logic_vector(2 downto 0);

--    RxBC0Rdy : out std_logic;

    -- TTC PHYSICAL
    SFP0RxN  : in std_logic;
    SFP0RxP  : in std_logic;
    SFP1RxN  : in std_logic;
    SFP1RxP  : in std_logic;

    SFP_LOS  : in std_logic;             -- Detects Rx Loss errors
    SFP_Rate  : out std_logic;            -- Selects Lower Rate Bandwidth
    
    SFP0TxN  : out std_logic;
    SFP0TxP  : out std_logic;
    SFP1TxN  : out std_logic;
    SFP1TxP  : out std_logic;

    SFP_TxDis  : out std_logic;            -- Disable Tx when High
    SFP_Fault  : in std_logic;           -- Tx Fault Detected
    SFP_P  : in std_logic;               -- Detects Presence of SFP Module

    -- TTC I2C Interface

    SFP_SCL  : out std_logic;
    SFP_SDA   : inout std_logic

    );
end TTCInterface;

architecture Behavioral of TTCInterface is
  -- Clock DCM Signals
  --component clk_dcm
  --  port
  --    (                                 -- Clock in ports
  --      clk_in  : in  std_logic;
  --      -- Clock out ports
  --      CLKx8   : out std_logic;
  --      CLKx8_n : out std_logic;

  --      CLKx4  : out std_logic;
  --      CLKx2  : out std_logic;
  --      CLKx1  : out std_logic;
  --      CLKx1_out : out std_logic;
  --      -- Status and control signals
  --      RESET  : in  std_logic;
  --      LOCKED : out std_logic
  --      );
  --end component;

  -- SFP Physical Signals/Buffers
  signal SFP0Rx : std_logic;                        -- Single Ended Rx
  signal SFP0RxQ : std_logic_vector (1 downto 0);

  signal SFP1Rx : std_logic;                        -- Single Ended Rx
  signal SFP1RxQ : std_logic_vector (1 downto 0);

  -- Single Ended Tx, shared between SFP 0 and SFP 1, both output channels are
  -- the same. Need Rx based arbitration for state. Just basic priority
  -- arbitration. Check with Eric on the arbitration technique
  signal SFP_Tx : std_logic;                        -- Single Ended Tx
            

  
  -- TTC Tx Data Signals
  signal TxTTC_ChSel : std_logic := '0';
  signal TxTTC_data : std_logic;
  component bmc_encoder
    port(
      data         : in  std_logic;
      clk160        : in  std_logic;
      rst_n     : in  std_logic;
      encoded_data : out std_logic
      );
  end component;

  --- TTC Tx BC0/ChB stuff
  COMPONENT ttc_bc0_transmitter
    PORT(
         clk40 : IN  std_logic;
         rst_n : IN  std_logic;
         orbit_enable : IN  std_logic;
         bc0_d : IN  std_logic_vector(7 downto 0);
         bc0_wr : IN  std_logic;
         bc0_sent : OUT  std_logic;
         orbit_strobe : OUT  std_logic;
         orbit_count : out unsigned (31 downto 0);
         bc0_out : OUT  std_logic
        );
    END COMPONENT;
    
  signal TxChB : std_logic := '0';
   --Inputs
  signal bc0_d : std_logic_vector(7 downto 0) := (others => '0');
  signal bc0_wr : std_logic := '0';

  --Outputs
  signal bc0_sent : std_logic;
  signal orbit_strobe : std_logic;
  signal bc0_out : std_logic;
  
  -- TTC Rx Data Signals
  --- RXTTC_data represents the raw decoded data from the SFP Rx Interface
  signal Rx0TTC_data_rdy : std_logic;
  signal Rx0TTC_data : std_logic;
  signal Rx0TTC_data_clk : std_logic;
  signal Rx0TTC_chB : std_logic;
  signal Rx0TTC_chB_clk : std_logic;

  signal Rx1TTC_data_rdy : std_logic;
  signal Rx1TTC_data : std_logic;
  signal Rx1TTC_data_clk : std_logic;
  signal Rx1TTC_chB : std_logic;
  signal Rx1TTC_chB_clk : std_logic;

  component bmc_decoder_ddr2
    port (
      -- Inputs
      d_in     : in std_logic_vector (1 downto 0);
      clk      : in std_logic;  -- Some clock that is preferably 4x the datarate clock
                                -- We will sync up to this clock. Since we are getting
                                      -- a ddr input we are at about 8x the input clock.
      enable_n : in std_logic;          -- Low Active Enable
      rst_n    : in std_logic;          -- Low Active Reset

      -- Outputs
      data       : out std_logic;
      data_ready : out std_logic;
      dclk_en      : out std_logic
      );
  end component;

  component ttc_demux
    port (
      -- Inputs
      input_stream : in std_logic;
      clk          : in std_logic;
      clk_en       : in std_logic;
      rst_n        : in std_logic;      -- Low Active Reset

      -- Outputs
      chA_data : out std_logic;
      chA_rdy  : out std_logic;
      chB_data : out std_logic;
      chB_rdy  : out std_logic
      );
  end component;

  component ttc_chb
    port (
      data           : in  std_logic;
      clk            : in  std_logic;
      rst_n          : in  std_logic;
--      enable_n       : in  std_logic;
      broadcast_data : out std_logic_vector (7 downto 0);
      frame_err      : out std_logic;
      single_err     : out std_logic;
      double_err     : out std_logic
      );
  end component;

  
begin
  -- Instantiate Clocks/DCM
  --TTCClockSystem001 : clk_dcm
  --  port map
  --  (                                   -- Clock in ports
  --    clk_in  => clk_in,
  --    -- Clock out ports
  --    CLKx8   => clk320,
  --    CLKx8_n => clk320_n,
  --    CLKx4   => clk160,
  --    CLKx2   => clk80,
  --    CLKx1   => clk40,
  --    CLKx1_out => clk_out,
  --    -- Status and control signals
  --    RESET   => '0',
  --    LOCKED  => open);
  
  -- Instantiate Physical Layer Connections for the TTC Stuff
  SFP_TxDis <= '0';
  SFP_Rate  <= '0';
    
  SFPRx_IBUFDS001 : IBUFDS
    generic map (
      DIFF_TERM    => true,             -- Differential Termination
      IBUF_LOW_PWR => false,  -- Low power (TRUE) vs. performance (FALSE) setting for referenced I/O standards
      IOSTANDARD   => "LVDS_33")
    port map (
      O  => SFP0Rx,
      I  => SFP0RxP,
      IB => SFP0RxN);

    SFPRx_IBUFDS002 : IBUFDS
    generic map (
      DIFF_TERM    => true,             -- Differential Termination
      IBUF_LOW_PWR => false,  -- Low power (TRUE) vs. performance (FALSE) setting for referenced I/O standards
      IOSTANDARD   => "LVDS_33")
    port map (
      O  => SFP1Rx,
      I  => SFP1RxP,
      IB => SFP1RxN);

  SFPRx_IDDR2001 : IDDR2
    generic map(
      -- Align with C0. Q[0] becomes the newest bit, Q[1] becomes the older bit
      DDR_ALIGNMENT => "C0",    -- Sets output alignment to "NONE", "C0", "C1"
      INIT_Q0       => '0',  -- Sets initial state of the Q0 output to '0' or '1'
      INIT_Q1       => '0',  -- Sets initial state of the Q1 output to '0' or '1'
      SRTYPE        => "SYNC")  -- Specifies "SYNC" or "ASYNC" set/reset
    port map (
      Q0 => SFP0RxQ(0),                -- 1-bit output captured with C0 clock
      Q1 => SFP0RxQ(1),                -- 1-bit output captured with C1 clock
      C0 => clk320,                      -- 1-bit clock input
      C1 => clk320_n,                    -- 1-bit clock input
      CE => '1',                        -- 1-bit clock enable input

      D => SFP0Rx,  -- Get input from output of Diff Pair Input Buffer

      R => '0',                         -- 1-bit reset input
      S => '0'                          -- 1-bit set input
      );

    SFPRx_IDDR2002 : IDDR2
    generic map(
      -- Align with C0. Q[0] becomes the newest bit, Q[1] becomes the older bit
      DDR_ALIGNMENT => "C0",    -- Sets output alignment to "NONE", "C0", "C1"
      INIT_Q0       => '0',  -- Sets initial state of the Q0 output to '0' or '1'
      INIT_Q1       => '0',  -- Sets initial state of the Q1 output to '0' or '1'
      SRTYPE        => "SYNC")  -- Specifies "SYNC" or "ASYNC" set/reset
    port map (
      Q0 => SFP1RxQ(0),                -- 1-bit output captured with C0 clock
      Q1 => SFP1RxQ(1),                -- 1-bit output captured with C1 clock
      C0 => clk320,                      -- 1-bit clock input
      C1 => clk320_n,                    -- 1-bit clock input
      CE => '1',                        -- 1-bit clock enable input

      D => SFP1Rx,  -- Get input from output of Diff Pair Input Buffer

      R => '0',                         -- 1-bit reset input
      S => '0'                          -- 1-bit set input
      );

  SFP0Tx_OBUFDS001 : OBUFDS
    generic map (
      IOSTANDARD => "LVDS_33"
      )
    port map(
      I  => SFP_Tx,                     -- Buffer output
      O  => SFP0TxP,
      OB => SFP0TxN);
  
  SFP0Tx_OBUFDS002 : OBUFDS
  generic map (
    IOSTANDARD => "LVDS_33"
  )
    port map(
      I  => SFP_Tx,                     -- Buffer output
      O  => SFP1TxP,
      OB => SFP1TxN);

  -----------------------------------------------------------------------------
  -- TTC I2C Interface                                                       --
  -----------------------------------------------------------------------------
  SFP_SCL <= '0';
  SFP_SDA <= 'Z';
  
  -----------------------------------------------------------------------------
  -- Tx TTC Interface Fabric                                                 --
  -----------------------------------------------------------------------------
  -- Generate the output data stream that needs to be encoded.
  TxTTCMux001 : process(clk80)
  begin
    if rising_edge(clk80) then
      if rst_n = '0' then
        TxTTC_data <= '0';
      else
        if TxTTC_ChSel= '1' then
          TxTTC_data <= TxL1A;
        else
          TxTTC_data <= TxChB;
        end if;
      end if;
    end if;
  end process;                     -- TXTTCMux001            

  -- Produce the Channel Select signal, toggling at 80Mhz
  TxTTCChSel001 : process(clk80)
  begin
    if rising_edge(clk80) then
      if rst_n = '0' then
        TxTTC_ChSel <= '0';
      else
        TxTTC_ChSel <= not(TxTTC_ChSel);
      end if;
    end if;
  end process;                     -- TXTTCChSel001

  -- NOTE: RENAME BLOCK to TTCBroadcast or ttx_broadcast
  TxTTCBroadcast001: ttc_bc0_transmitter PORT MAP (
    clk40 => clk40,
    rst_n => rst_n,
    orbit_enable => TxBC0_orbit_en,

    bc0_d => TxBC0,
    bc0_wr => TxBC0_wr,
    bc0_sent => TxBC0_sent,
    orbit_strobe => TxBC0_orbit_strobe,
    orbit_count => TxBC0_orbit_ctr,
    bc0_out => TxChB
    );

  -- Encodes Output TTC Data to be sent over the TTC (SFP) Interface.
  BiphaseMarkEncoder001: bmc_encoder port map (
    data => TxTTC_data,--data,
    clk160 => clk160,
    rst_n => rst_n,
    encoded_data => SFP_Tx
    );

  -----------------------------------------------------------------------------
  -- Rx TTC Interface Fabric                                                 --
  -----------------------------------------------------------------------------
  RxBMCDecoderDDR2001 : bmc_decoder_ddr2 port map (
    -- Inputs
    d_in       => SFP0RxQ,
    clk        => clk320,
    enable_n   => enable_n,
    rst_n      => rst_n,
    data       => Rx0TTC_data,
    data_ready => Rx0TTC_data_rdy,
    dclk_en      => Rx0TTC_data_clk
    );

  RxBMCDecoderDDR2002 : bmc_decoder_ddr2 port map (
    -- Inputs
    d_in       => SFP1RxQ,
    clk        => clk320,
    enable_n   => enable_n,
    rst_n      => rst_n,
    data       => Rx1TTC_data,
    data_ready => Rx1TTC_data_rdy,
    dclk_en      => Rx1TTC_data_clk
    );

  RxTTCDemux001 : ttc_demux port map (
    -- Inputs
    input_stream => Rx0TTC_data,
    clk => clk160,
    -- clk          => RxTTC_data_clk,
    clk_en => Rx0TTC_data_clk,
    rst_n        => rst_n,              -- Low Active Reset
    
    -- Outputs
    chA_data => Rx0L1A,
    chA_rdy  => Rx0L1A_clk,
    chB_data => Rx0TTC_chB,
    chB_rdy  => Rx0TTC_chB_clk
    );

    RxTTCDemux002 : ttc_demux port map (
    -- Inputs
    input_stream => Rx1TTC_data,
    clk => clk160,
    -- clk          => RxTTC_data_clk,
    clk_en => Rx1TTC_data_clk,
    rst_n        => rst_n,              -- Low Active Reset
    
    -- Outputs
    chA_data => Rx1L1A,
    chA_rdy  => Rx1L1A_clk,
    chB_data => Rx1TTC_chB,
    chB_rdy  => Rx1TTC_chB_clk
    );

   Rx0TTCChB : ttc_chb port map (
    data           => Rx0TTC_chB,
    clk            => Rx0TTC_chB_clk,
    rst_n          => rst_n,            -- Low Active Reset

    broadcast_data => Rx0BC0,

    frame_err      => Rx0BC0Err(0),
    single_err     => Rx0BC0Err(1),
    double_err     => Rx0BC0Err(2)
    );

    Rx1TTCChB : ttc_chb port map (
    data           => Rx1TTC_chB,
    clk            => Rx1TTC_chB_clk,
    rst_n          => rst_n,            -- Low Active Reset

    broadcast_data => Rx1BC0,

    frame_err      => Rx1BC0Err(0),
    single_err     => Rx1BC0Err(1),
    double_err     => Rx1BC0Err(2)
    );

end Behavioral;

