----------------------------------------------------------------------------------
-- Company: Boston University EDF
-- Engineer: Christopher Woodall
-- 
-- Create Date:    12:38:52 06/05/2012 
-- Design Name: TTS/TTC Adapter Phase 2 Firmware
-- Module Name:    top_module - Behavioral
-- Project Name: TTS/TTC Adapter
-- Target Devices: XC6SLX9-TQFP144
-- Tool versions: 13.3 (lin64)
-- Description: 
--
-- Dependencies: 
--
-- Revision: Phase2 Firmware
-- Revision 0.1 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;
use work.ipbus.all;
use work.bus_arb_decl.all;
use IEEE.STD_LOGIC_MISC.all;

-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
use IEEE.NUMERIC_STD.all;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity top_module is
  port (

    -- SFP 0 
    SFP0RxN   : in std_logic;
    SFP0RxP   : in std_logic;
    SFP0LOS   : in std_logic;
    SFP0Rate  : out std_logic;
    SFP0TxN   : out std_logic;
    SFP0TxP   : out std_logic;
    SFP0TxDis : out std_logic;
    SFP0Fault : in  std_logic;
    SFP0P     : in std_logic;
    SFP0SDA   : inout std_logic;
    SFP0SCL   : out std_logic;

    -- SFP 1
    SFP1RxN     : in std_logic;
    SFP1RxP     : in std_logic;
    SFP1LOS     : in std_logic;
    SFP1Rate    : out std_logic;
    SFP1TxN     : out std_logic;
    SFP1TxP     : out std_logic;
    SFP1TxDis   : out std_logic;
    SFP1Fault   : in std_logic;
    SFP1P       : in std_logic;
    SFP1SDA     : inout std_logic;
    SFP1SCL     : out std_logic;

    -- TTS RJ-45 I/O
    TTSA_Tx : out std_logic_vector (3 downto 0);  -- Channel A Out
    TTSA_Rx : in  std_logic_vector (3 downto 0);  -- Channel A In
    TTSB_Tx : out std_logic_vector (3 downto 0);  -- Channel B Out
    TTSB_Rx : in  std_logic_vector (3 downto 0);  -- Channel B In

    -- NIM I/O
    NimOut : out std_logic_vector (3 downto 0);
    NimIn  : in std_logic_vector (3 downto 0);

    -- Ethernet Inteface (should change signal names)
    EthRxD      : in    std_logic_vector(3 downto 0);
    EthRxDv      : in    std_logic;
    EthRxClk     : in    std_logic;
    EthRxEr      : in    std_logic;
    EthTxD       : out   std_logic_vector(3 downto 0);
    EthTxEn      : out   std_logic;
    EthTxEr      : out   std_logic;
    EthTxClk     : in    std_logic;
    EthCrs       : in    std_logic;
    EthCol       : in    std_logic;
    EthRst_n     : out   std_logic;
    EthMdio      : inout std_logic;
    EthMdc       : out   std_logic;

    -- LED Interfaces
    led_err : out std_logic;  -- Single Error LED.
    led_clk : out std_logic;  -- Clock for 8 shift register bound LEDs
    led_d   : out std_logic;  -- Data for 8 shift register bounde LEDs

    -- Clock
    clk_in : in std_logic;                 -- Clock input from ~40MHz Oscillator

    -- Jumper Select Pins
    MacSel : in std_logic_vector(3 downto 0);

    -- Buttons
--    SoftReset_N : in std_logic;
--    TriggerButton_N : in std_logic;
    
    -- FLASH Interfaces
    FLASH_S : out  STD_LOGIC;
    FLASH_D : out  STD_LOGIC;
    FLASH_Q : in  STD_LOGIC;
    FLASH_C : out  STD_LOGIC
    );
end top_module;

architecture Behavioral of top_module is
  signal clk : std_logic := '0';
  
  component led_shifter
    port(
      leds    : in  std_logic_vector(7 downto 0);
      latch   : in  std_logic;
      clk     : in  std_logic;
      led_d   : out std_logic;
      led_clk : out std_logic
      );
  end component;
  signal leds  : std_logic_vector (7 downto 0) := x"00";
  signal latch : std_logic                     := '0';

  signal enable_n   : std_logic := '1';
  signal rx_mac_clk : std_logic;
  signal rx_mac_clk_en : std_logic;
  
  signal rst_n : std_logic := '0';
  signal rst   : std_logic := '1';

  --testing clks
  signal rx_mac_valid     : std_logic;
  signal rx_mac_badframe  : std_logic;
  signal rx_mac_goodframe : std_logic;
  signal rx_mac_data      : std_logic_vector(7 downto 0);
  signal acked         : std_logic               := '0';
  signal tx_mac_clk_en : std_logic;
  signal tx_mac_clk    : std_logic;
  signal tx_mac_valid  : std_logic               := '0';
  signal tx_mac_ack    : std_logic;
  signal tx_mac_data   : std_logic_vector(7 downto 0);
  signal tx_packet_ctr : integer range 0 to 2000000;
  signal addr_ctr      : integer range 0 to 4096 := 0;
  signal packets_rx    : integer range 0 to 400  := 0;

  -- IPBUS
  signal locked : std_logic := '0';
  signal ipb_clk         : std_logic;


  signal ipb_master_out : ipb_wbus;
  signal ipb_master_in  : ipb_rbus;
  signal mac_addr       : std_logic_vector(47 downto 0);
  signal ip_addr        : std_logic_vector(31 downto 0);
  signal oob_moti       : trans_moti;
  signal oob_tomi       : trans_tomi;
--  signal gpio           : std_logic_vector (3 downto 0);

  -- DCM Clock Generator (80MHz clk and 4 phase 40MHz clk)
  component tts_clk_gen
    port
      (-- Clock in ports
        CLK_IN           : in     std_logic;
        -- Clock out ports
        CLK40 : out std_logic;
        --CLK40_0         : out    std_logic;
        --CLK40_90          : out    std_logic;
        --CLK40_180          : out    std_logic;
        --CLK40_270          : out    std_logic;
        CLK80          : out    std_logic;
        -- Status and control signals
        RESET             : in     std_logic;
        LOCKED            : out    std_logic
        );
  end component;

  -- TTS Control Fabric
  component tts_control
    port (
      parm    : in  std_logic_vector(255 downto 0);
      orb_clk : in  std_logic_vector(3 downto 0);
      clk80   : in  std_logic;
      tts_a   : in  std_logic_vector(3 downto 0);
      cr      : in  std_logic_vector(7 downto 0);
      ar      : in  std_logic_vector(7 downto 0);
      sr      : out std_logic_vector(7 downto 0);
      dr      : out std_logic_vector(7 downto 0);
      bx_count : in std_logic_vector(11 downto 0);
      nim_l1a : out std_logic;
      nim_ttc : out std_logic;
      ecl_bc0 : out std_logic;
      test : out std_logic_vector(1 downto 0));
  end component;

  signal parm    : std_logic_vector(255 downto 0);
  signal orb_clk : std_logic_vector(3 downto 0);

  signal clk80   : std_logic;
  signal clk320 : std_logic;
  signal clk320_n :std_logic;
  signal clk160 : std_logic;
  
  signal tts_a   : std_logic_vector(3 downto 0);
  signal cr      : std_logic_vector(7 downto 0);
  signal ar      : std_logic_vector(7 downto 0);
  signal sr      : std_logic_vector(7 downto 0);
  signal dr      : std_logic_vector(7 downto 0);
  signal nim_l1a : std_logic;
  signal nim_ttc : std_logic;
  signal ecl_bc0 : std_logic;
  signal clk_sys :std_logic;

  --signal TxL1A              : std_logic;
  --signal TxBC0              : std_logic_vector(7 downto 0);
  --signal TxBC0_wr           : std_logic;
  --signal TxBC0_sent         : std_logic;
  --signal TxBC0_orbit_strobe : std_logic;
  --signal TxBC0_orbit_en     : std_logic;
  --signal TxBC0_orbit_ctr    : unsigned(31 downto 0);
  --signal RxL1A              : std_logic;
  --signal RxL1A_clk          : std_logic;
  --signal RxBC0              : std_logic_vector(7 downto 0);
  --signal RxBC0Err           : std_logic_vector(2 downto 0);

  signal TxBC0              : std_logic_vector(7 downto 0) := x"00";
  signal TxBC0_wr           : std_logic                    := '0';
  signal TxBC0_orbit_en     : std_logic                    := '0';
  signal TxBC0_orbit_strobe : std_logic;
  signal TxBC0_sent         : std_logic;
  signal TxBC0_orbit_ctr    : unsigned (31 downto 0)       := x"00000000";
  signal L1Acounter         : integer range 0 to 20000 := 0;

  signal TxL1A : std_logic;
  signal L1Alimit : integer range 0 to 11250;
  signal Rx0L1A    : std_logic;
  signal Rx0L1A_Clk    : std_logic;
  signal Rx1L1A    : std_logic;
  signal Rx1L1A_Clk    : std_logic;

  signal resetttc :std_logic;
  signal L1Ajumpers :std_logic_vector (1 downto 0) := b"10";
  --signal RxBcast    : std_logic_vector(7 downto 0);
  --signal RxBcast_strobe : std_logic;
  signal Rx0BC0 :Std_logic_vector (7 downto 0);
  signal Rx1BC0 :Std_logic_vector (7 downto 0);  
  signal Rx0BcastErr : std_logic_vector(2 downto 0);
  signal Rx1BcastErr : std_logic_vector(2 downto 0);

  --signal RxBcastAnyErr :std_logic;
  signal reset_sequence_sent : std_logic;
  --signal RxBcast_ctr : unsigned(7 downto 0);
  --signal RxBcast_strobe_p : std_logic;
  --signal RxBcastErr_ctr : unsigned(3 downto 0);
  
  signal ResetTTC_p : std_logic;
  signal ResetTTC_counter : integer range 0 to 10000;
  signal resetTTC_db : std_logic;
  
  signal txclk : std_logic;
  signal rxclk : std_logic;
  signal txclk_b :std_logic;
  signal rxclk_b :std_logic;
  signal rx_mac_clk_en_phase : std_logic;
  signal tx_mac_clk_en_phase : std_logic;

  signal rst_dcm_n : std_logic := '1';  
  signal l1a_test : std_logic_vector (7 downto 0);
  signal ipb_clk_n : std_logic;
  
  signal rdata : std_logic_vector(31 downto 0) := (others => '0');
  signal   flash_wdata            : std_logic_vector(31 downto 0) := (others => '0');
  signal   flash_rdata            : std_logic_vector(31 downto 0) := (others => '0');
  signal   flash_busy             : std_logic                     := '0';
  signal   flash_md               : std_logic                     := '0';
  signal   flash_mc               : std_logic                     := '0';
  signal   flash_mcs              : std_logic_vector(1 downto 0)  := (others => '1');


  -----------------------------------------------------------------------------
  -- Test Signals
  -----------------------------------------------------------------------------
  signal test_en : std_logic := '0';    -- Test Enable, when set to 1 on boot
                                        -- up test mode is activated
  signal test_counter : unsigned(31 downto 0) := x"00000000";
  signal test_leds : std_logic_vector (7 downto 0);
  signal NimOut_reg : std_logic_vector (3 downto 0);
  signal NimRead : std_logic_vector ( 3 downto 0);
  signal pulse_counter : unsigned (31 downto 0);
begin

  EthRst_n  <= rst_n;                    -- Sync Ethernet reset
  rst       <= not(rst_n);              -- high active reset signal

--  NimL1A <= nim_l1a;
--  NimBC0 <= nim_ttc;

  
  --  ipb_clk <= tx_mac_clk;
  ipb_clk_n <= not(ipb_clk);
  generate_ipb_clk : process(tx_mac_clk)
    variable counter : unsigned (1 downto 0) := "00";
  begin
    if rising_edge(tx_mac_clk) then
--      if (rst_n = '1') then
        ipb_clk <= counter(1);
--        ipb_clk_n <= not( counter(0) );
        counter := counter + 1;
--      end if;
    end if;
  end process;

  -- Trying to move to a more reasonable method of dividing the clock...
  -- Tons of errors result... Fix them all.
  BUFIO2_inst : BUFIO2
    generic map (
      DIVIDE => 2, -- Set the DIVCLK divider divide-by value.
      DIVIDE_BYPASS => FALSE, -- DIVCLK output sourced from Divider (FALSE) or from I input, bypassing Divider
-- (TRUE).
      I_INVERT => FALSE,
      USE_DOUBLER => FALSE
      )
    port map (
      DIVCLK => rxclk_b,-- rx_mac_clk, -- 1-bit Output divided clock
      IOCLK => open, -- 1-bit Output clock
      SERDESSTROBE => open, -- 1-bit Output SERDES strobe (Clock Enable)
      I => EthRxClk -- 1-bit Clock input
      );

  BUFG_inst : BUFG
    port map(
      O => rx_mac_clk,
      I => rxclk_b
      );

  generateClkEnablesRx: process (EthRxClk)
    variable p : std_logic;
  begin
    if rising_edge(EthRxClk) then
      if rx_mac_clk_en_phase = '0' and p = '0' then
        p := '1';
      else
        rx_mac_clk_en <= not rx_mac_clk_en;
        p := '0';
      end if;
    end if;
  end process;

  fixClkEnablesRxPhase: process (rx_mac_clk)
  begin
    if rising_edge(rx_mac_clk) then
      if rx_mac_clk_en = '0' then
        rx_mac_clk_en_phase <= '0';       
      else
        rx_mac_clk_en_phase <= '1';
      end if;
    end if;
  end process;
   
    BUFIO2_tx_maclk_inst : BUFIO2
    generic map (
      DIVIDE => 2, -- Set the DIVCLK divider divide-by value.
      DIVIDE_BYPASS => FALSE, -- DIVCLK output sourced from Divider (FALSE) or from I input, bypassing Divider
-- (TRUE).
      I_INVERT => FALSE,
      USE_DOUBLER => FALSE
      )
    port map (
      DIVCLK => txclk_b,--tx_mac_clk, -- 1-bit Output divided clock
      IOCLK => open, -- 1-bit Output clock
      SERDESSTROBE => open, -- 1-bit Output SERDES strobe (Clock Enable)
      I => EthTxClk -- 1-bit Clock input
      );

    BUFG1_inst : BUFG
    port map(
      O => tx_mac_clk,
      I => txclk_b
      );

  generateClkEnablesTx: process (EthTxClk)
    variable p : std_logic;
  begin
    if rising_edge(EthTxClk) then
     if tx_mac_clk_en_phase = '0' and p = '0' then
       p := '1';
     else
        tx_mac_clk_en <= not tx_mac_clk_en;
       p := '0';
     end if;
    end if;
  end process;
  
  fixClkEnablesTxPhase: process(tx_mac_clk)
  begin
    if rising_edge(tx_mac_clk) then
      if tx_mac_clk_en = '0' then
        tx_mac_clk_en_phase <= '0';       
      else
        tx_mac_clk_en_phase <= '1';
      end if;
    end if;
  end process;

  
  OEthMac001 : entity work.ethernet_mac
    port map (
    rxd              => EthRxD,
    rxdv             => EthRxDv,
    rxclk            => EthRxClk,
--    rxclk => rxclk,
    
    rxer             => EthRxEr,
    txd              => EthTxD,
    txen             => EthTxEn,
    txer             => EthTxEr,
    txclk            => EthTxClk,
    crs              => EthCrs,
    col              => EthCol,
    mdio             => EthMdio,
    mdc              => EthMdc,

    rx_mac_clk       => rx_mac_clk,
    rx_mac_clk_en    => rx_mac_clk_en,
    rx_mac_valid     => rx_mac_valid,
    rx_mac_data      => rx_mac_data,
    rx_mac_goodframe => rx_mac_goodframe,
    rx_mac_badframe  => rx_mac_badframe,

    tx_mac_clk       => tx_mac_clk,
    tx_mac_clk_en    => tx_mac_clk_en,
    tx_mac_valid     => tx_mac_valid,
    tx_mac_ack       => tx_mac_ack,
    tx_mac_data      => tx_mac_data,
    rst_n            => rst_n,
    enable_n         => enable_n
    );

  -- ipbus control logic
  -- seems taht rst_macclk is high active
  ipbus : entity work.ipbus_ctrl
    generic map(NOOB => 1)
    port map(
      ipb_clk         => ipb_clk,
      rst_ipb         => rst,
      rst_macclk      => rst,
      mac_txclk       => tx_mac_clk,
      mac_rxclk       => rx_mac_clk,
      mac_rxd         => rx_mac_data,
      mac_rxdvld      => rx_mac_valid,
      mac_rxgoodframe => rx_mac_goodframe,
      mac_rxbadframe  => rx_mac_badframe,
      mac_txd         => tx_mac_data,
      mac_txdvld      => tx_mac_valid,
      mac_txack       => tx_mac_ack,
      ipb_out         => ipb_master_out,
      ipb_in          => ipb_master_in,
      mac_addr        => mac_addr,
      ip_addr         => ip_addr,
      oob_moti_bus(0) => oob_moti,
      oob_tomi_bus(0) => oob_tomi
      );


  --ipbus slaves live in the entity below, and can expose top-level ports
  --The ipbus fabric is instantiated within.

  slaves : entity work.slaves port map(
    ipb_clk  => ipb_clk,
    rst      => rst,
    ipb_in   => ipb_master_out,
    ipb_out  => ipb_master_in,
    -- Top level ports from here
    parm => parm,
    cr => cr,
    ar => ar,
    dr => dr,
    sr => sr,
    l1a_test => l1a_test,
    oob_moti => oob_moti,
    oob_tomi => oob_tomi,

    flash_busy  => flash_busy,
    flash_wdata => flash_wdata,
    flash_rdata => flash_rdata,
    
    MacSel => MacSel
    );

  led_err <= locked; --or and_reduce(cr & ar & parm & '0');

  -- LED Display  
  LEDShift : led_shifter
    port map (
      leds    => leds,
      latch   => latch,
      clk     => clk,
      led_d   => led_d,
      led_clk => led_clk);

--  led_err <= tx_mac_clk;
  led_display : process (clk)
    variable counter         : unsigned (31 downto 0) := x"00000000";
    variable pattern_counter : integer range 0 to 7   := 0;
  begin
    if rising_edge(clk) then
      if (rst_n = '1') then
        counter := counter + 1;
        latch   <= counter(10);
        
        if (test_en = '1') then
          leds <= test_leds;
        else
          leds <= Rx0BC0(7 downto 4) & MacSel; --std_logic_vector(counter(27 downto 24));
        end if;

      end if;
    end if;
  end process;

  --TTSA_Tx <= RxBC0(7 downto 4);
  ---- Ordering for TTS Subsection written by Eric Hazen:
  ----    OVF = 0001 => 0100 
  ----    SYN = 0010 => 1000
  ----    BSY = 0100 => 0001
  ----    RDY = 1000 => 0010
  --tts_a <= RxBC0(7 downto 4);
  --tts_control_1: tts_control
  --  port map (
  --    parm    => parm,
  --    orb_clk => orb_clk,
  --    clk80   => clk80, 
  --    tts_a   => tts_a,
  --    cr      => cr,
  --    ar      => ar,
  --    sr      => sr,
  --    dr      => dr,
  --    nim_l1a => nim_l1a,
  --    nim_ttc => nim_ttc,
  --    bx_count => std_logic_vector(TxBC0_orbit_ctr(11 downto 0)),
  --    ecl_bc0 => ecl_bc0,
  --    test    => open);

  clk_dcm : entity work.clk_dcm
    port map (
      clk_in => clk_in,
      clkx8 => clk320,
      clkx8_n => clk320_n,
      clkx4 => clk160,
      clkx2 => clk80,
      clkx1 => clk,
--      clkx1_out => open,
      reset => '0',
      locked => locked
    );

  -- Generate 4 Phase Clock
  orb_clk(0) <= clk;
  orb_clk(1) <= clk;
  orb_clk(2) <= clk;
  orb_clk(3) <= clk;


  
  process (clk)
  begin
    if rising_edge(clk) then
      if (pulse_counter > to_unsigned(5000, 32)) and (pulse_counter < to_unsigned(5021,32)) then
        NimOut <= NimOut_reg;
      else
        NimOut <= x"0";
      end if;
    end if;
  end process;    

  process (clk)
  begin
    if rising_edge(clk) then
      if (pulse_counter = to_unsigned(5010, 32)) then
        NimRead <= not(NimIn);
      end if;
    end if;
  end process;    
  
  TestSignals : process (clk)
  begin
    if test_en = '1' then
      if rising_edge(clk) then
        case MacSel is
          when x"0" => NimOut_reg <= std_logic_vector(test_counter(27 downto 24));
                       test_leds(3 downto 0) <= NimRead;
                       test_leds(7 downto 4) <= x"0";
          when x"1" => TTSA_Tx <= std_logic_vector(test_counter(31 downto 28));
                       TTSB_Tx <= std_logic_vector(test_counter(27 downto 24));
                       test_leds(7 downto 4) <= TTSA_Rx;
                       test_leds(3 downto 0) <= TTSB_Rx;
          when x"2" => test_leds <= Rx0BC0;
          when x"4" => test_leds <= Rx1BC0;
          when others =>
            if (test_counter(25) = '1') then
              test_leds <= x"FF";
            else
              test_leds <= x"00";
            end if;
            
        end case;      
      end if;   
    end if;
  end process;
  
  TestCounter : process (clk)
  begin
    if test_en = '1' then
      if rising_edge(clk) then
        test_counter <= test_counter + 1;

        if (pulse_counter < 5022) then
          pulse_counter <= pulse_counter + 1;
        else
          pulse_counter <= x"00000000";
        end if;
      end if;
    end if;
  end process;

  reset_delay : process(clk)
    variable counter : unsigned (31 downto 0) := x"00000000";
  begin
    if rising_edge(clk) then
      if (rst_n = '0') then
        -- Set MacAddress and IPAddress
        mac_addr <= X"080030F300" & x"1" & MacSel;  -- Mac Addresses between
                                               -- 08-00-30-F3-00-10 and
                                               -- 08-00-30-F3-00-2F if all
                                               -- possible positions were
                                               -- used.

        ip_addr  <= X"c0a802" & X"2" & MacSel;  -- IP Addresses between
                                              -- 192.168.2.32 and 192.168.2.63
                                              -- Correspond directly to MAC
                                              -- Addresses
        
        if MacSel = x"0" then
          test_en <= '1';
        else
          test_en <= '0';
        end if;
        
        counter := counter + 1;
        rst_n   <= counter(10);
      else
        counter := x"00000000";
      end if;
    end if;
  end process;
  enable_n <= '0';
  

  -- Instantiate Module and signals
  -- Also can drive multiple DCMs from clk_in if I instantatiate my own IBUFG
  TTCInterface_001: entity work.TTCInterface
    port map (
      --clk_in             => clk_in,
      --clk_out            => open,
      clk80 => clk80,
      clk320_n => clk320_n,
      clk320 => clk320,
      clk160 => clk160,
      clk40 => clk,
      rst_n              => rst_n,
      enable_n          => '0',
      TxL1A              => TxL1A,
      TxBC0              => TxBC0,
      TxBC0_wr           => TxBC0_wr,
      TxBC0_sent         => TxBC0_sent,
      TxBC0_orbit_strobe => TxBC0_orbit_strobe,
      TxBC0_orbit_en     => TxBC0_orbit_en,
      TxBC0_orbit_ctr    => TxBC0_orbit_ctr,
      Rx0L1A              => Rx0L1A,
      Rx0L1A_clk          => Rx0L1A_clk,
      Rx1L1A              => Rx1L1A,
      Rx1L1A_clk          => Rx1L1A_clk,
      Rx0BC0              => Rx0BC0,
      Rx0BC0Err           => Rx0BcastErr,
      Rx1BC0              => Rx1BC0,
      Rx1BC0Err           => Rx1BcastErr,

      SFP0RxN            => SFP0RxN,
      SFP0RxP            => SFP0RxP,
      SFP1RxN            => SFP1RxN,
      SFP1RxP            => SFP1RxP,

      SFP_LOS            => SFP0LOS,
      SFP_Rate           => SFP0Rate,
      SFP0TxN            => SFP0TxN,
      SFP0TxP            => SFP0TxP,
      SFP1TxN            => SFP1TxN,
      SFP1TxP            => SFP1TxP,

      SFP_TxDis          => SFP0TxDis,
      SFP_Fault          => SFP0Fault,
      SFP_P              => SFP0P,
      SFP_SCL            => SFP0SCL,
      SFP_SDA            => SFP0SDA
    );
  
  broadcast : process (clk)
    variable startup_ctr : integer range 0 to 40800000 := 0;
    variable ecr_sent    : std_logic                   := '0';
    variable ocr_sent    : std_logic                   := '1';
  begin
    if rising_edge(clk) then
      if rst_n = '0' then
        startup_ctr    := 0;
  --      TxBC0_orbit_en <= '0';
        TxBC0_wr       <= '0';
        TxBC0          <= x"00";
        ecr_sent       := '0';
        ocr_sent       := '0';
        reset_sequence_sent <= '0';
      else
        if (resetTTC_db = '1') then
          startup_ctr := 0;
          reset_sequence_sent <= '0';
          ocr_sent := '0';
          ecr_sent := '0';
        end if;
        
        if startup_ctr < 40000000 then
          startup_ctr := startup_ctr + 1;
        else
          TxBC0 <= std_logic_vector(test_counter(31 downto 24));
          if TxBC0_sent = '0' then
            TxBC0_wr <= '1';
          else
            TxBC0_wr <= '0';
          end if;
        end if;
          --if ecr_sent = '0' then
          --  TxBC0 <= x"E2";
          --  if TxBC0_sent = '0' then
          --    TxBC0_wr <= '1';
          --  else
          --    TxBC0_wr    <= '0';
          --    ecr_sent    := '1';
          --    startup_ctr := startup_ctr - 16;
          --  end if;
          --elsif ocr_sent = '0' then
          --  TxBC0 <= x"28";
          --  if TxBC0_sent = '0' then
          --    TxBC0_wr <= '1';
          --  else
          --    TxBC0_wr    <= '0';
          --    ocr_sent    := '1';
          --    startup_ctr := startup_ctr - 40000000;
          --  end if;
          --else
          --  TxBC0_wr <= '0';
          --  reset_sequence_sent <= '1';
          --end if;
          --end if;
      end if;
    end if;
  end process;
  TxBC0_orbit_en <= '1';

  resetTTC_a: process(clk)
  begin
    if rising_edge(clk) then
      if rst_n = '0' then
        resetTTC_counter <= 0;
        resetTTC_db <= '0';
      else
--      resetTTC_db <= not resetTTC;
        if (resetTTC_counter = 0) then
          if (resetTTC_p /= resetTTC) then
            resetTTC_counter <= resetTTC_counter + 1;
          end if;
          resetTTC_db <= '0';
        elsif (resetTTC_counter >= 10000) then
          resetTTC_counter <= 0;
          if (resetTTC_p = resetTTC) then
            resetTTC_db <= not resetTTC;
          end if;
--        resetTTC_db <= not(resetTTC);
        else
          resetTTC_counter <= resetTTC_counter + 1;
        end if;
      end if;
    end if;
  end process;
  resetTTC <= '0';

  resetTTC_prev: process(clk)
  begin
    if rising_edge(clk) then
      resetTTC_p <= resetTTC;
    end if;
  end process;
  TxL1A <= '0';
  
  -- Implementing Flash
  process (ipb_clk)
  begin
    if (rising_edge(ipb_clk)) then
      FLASH_S <= not flash_MCS(1) and not flash_MCS(0);
      FLASH_D <= flash_MD;
    end if;
  end process;

  i_flash : entity work.flash port map(
    clk         => ipb_clk,
    we          => ipb_master_out.ipb_write,
    strobe      => ipb_master_out.ipb_strobe,
    w_data      => ipb_master_out.ipb_wdata,
    addr        => ipb_master_out.ipb_addr,
    flash_wdata => flash_wdata,
    flash_rdata => flash_rdata,
    flash_busy  => flash_busy,
    MQ          => FLASH_Q,
    MD          => flash_MD,
    MCS         => flash_MCS
    );

  i_FLASH_C_ddr : ODDR2
    port map (
      Q  => FLASH_C,                    -- 1-bit output data
      C0 => ipb_clk,                    -- 1-bit clock input
      C1 => ipb_clk_n,                  -- 1-bit clock input
      CE => '1',                        -- 1-bit clock enable input
      D0 => '0',           -- 1-bit data input (associated with C0)
      D1 => flash_MCS(1),  -- 1-bit data input (associated with C1)
      R  => '0',                        -- 1-bit reset input
      S  => '0'                         -- 1-bit set input
      );
  SFP1SDA <= '0';
  SFP1SCL <= '0';
  SFP1TxDis <= '0';
  SFP1Rate <= '0';

end Behavioral;

