--
--	Package File Template
--
--	Purpose: This package defines supplemental types, subtypes, 
--		 constants, and functions 
--
--   To use any of the example code shown below, uncomment the lines and modify as necessary
--

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.numeric_std.all;
use work.ipbus.all;
package ttt_funcs is

  function get_bits ( v : std_logic_vector; lo : integer; len : integer) return std_logic_vector;

  -- procedure for writing specific bits in a large vector based on,
  -- start position and length (lo and length)
  procedure write_bits(
    signal src : in std_logic_vector;
    signal dest : inout std_logic_vector;
    lo : in integer;
    len: in integer);

  --procedure ttt_ipb_rw_reg(
  --  signal ipbwr : in ipb_wbus;
  --  signal ipbrd : out ipb_rbus;
  --  signal reg : inout std_logic_vector;
  --  lo : in integer;
  --  len : in integer;
  --  wr_en : in std_logic);
  procedure ttt_ipb_rw_reg(
    signal ipbwr : in ipb_wbus;
    signal rdata : out std_logic_vector( 31 downto 0);
--  signal ipbrd : out ipb_rbus;
    signal  reg : inout std_logic_vector;
    lo : in integer;
    len : in integer;
    wr_en : in std_logic);
  procedure ttt_ipb_read_const(
    signal rdata: out std_logic_vector(31 downto 0);
    --signal ipbwr : in ipb_wbus;
    --signal ipbrd : out ipb_rbus;
    const : in std_logic_vector);

  procedure ttt_ipb_read_reg(
    signal rdata: out std_logic_vector(31 downto 0);
    --signal ipbwr : in ipb_wbus;
    --signal ipbrd : out ipb_rbus;
    signal reg : in std_logic_vector);
  
  constant TTTFirmwareVersion : std_logic_vector(31 downto 0) := x"20130114";

end ttt_funcs;

package body ttt_funcs is
 -- functions to access a byte in a large vector
  function get_bits ( v : std_logic_vector; lo : integer; len : integer)
    return std_logic_vector is
  begin
    return v( lo+len-1 downto lo);
  end function get_bits;

 -- procedure for writing specific bits in a large vector based on,
  -- start position and length (lo and length)
  procedure write_bits(
    signal src : in std_logic_vector;
    signal dest : inout std_logic_vector;
    lo : in integer;
    len: in integer)  is
  begin
    dest(lo+len-1 downto lo) <= src(len-1 downto 0);
  end write_bits;
  
  procedure ttt_ipb_rw_reg(
    signal ipbwr : in ipb_wbus;
    signal rdata : out std_logic_vector( 31 downto 0);
--  signal ipbrd : out ipb_rbus;
    signal  reg : inout std_logic_vector;
    lo : in integer;
    len : in integer;
    wr_en : in std_logic) is
  begin
    if ipbwr.ipb_write = '1' and ipbwr.ipb_strobe = '1' and wr_en = '1' then
      write_bits(ipbwr.ipb_wdata, reg, lo, len);
    end if;
    rdata <= std_logic_vector(resize(unsigned(get_bits(reg, lo, len)), 32));
    
    --ipbrd.ipb_ack <= ipbwr.ipb_strobe;
    --ipbrd.ipb_err <= '0';                     
  end ttt_ipb_rw_reg;

  procedure ttt_ipb_read_const(
    signal rdata : out std_logic_vector(31 downto 0);
    const : in std_logic_vector) is
  begin
    rdata <= const;
    
    --ipbrd.ipb_ack <= ipbwr.ipb_strobe;
    --ipbrd.ipb_err <= '0';                     
  end ttt_ipb_read_const;

  procedure ttt_ipb_read_reg(
    signal rdata: out std_logic_vector(31 downto 0);
    signal reg : in std_logic_vector) is
  begin
    rdata <= std_logic_vector(resize(unsigned(reg),32));
    
    --ipbrd.ipb_ack <= ipbwr.ipb_strobe;
    --ipbrd.ipb_err <= '0';                     
  end ttt_ipb_read_reg;

end ttt_funcs;
